﻿//
// BlankPage.xaml.cpp
// Implementation of the BlankPage.xaml class.
//

#include "pch.h"
#include "BlankPage.xaml.h"

#include <client.h>
#include <time.h>
#include <math.h>
#include <d2d1_1.h>
#include <d3d11_1.h>
#include <windows.ui.xaml.media.dxinterop.h>

using namespace Direct2D;

using namespace Platform;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Media::Imaging;
using namespace Windows::UI::Xaml::Navigation;
using namespace Microsoft::WRL;

// The Blank Page item template is documented at http://go.microsoft.com/fwlink/?LinkId=234238

const int PixelWidth = 1024;
const int PixelHeight = 768;
const int PointCount = 5000;

BlankPage::BlankPage()
{
	InitializeComponent();

	Application::Current->DebugSettings->EnableFrameRateCounter = true;

	this->imageSource = ref new SurfaceImageSource(PixelWidth, PixelHeight);
	IInspectable* inspectable = (IInspectable*) reinterpret_cast<IInspectable*>(this->imageSource);
	inspectable->QueryInterface(__uuidof(ISurfaceImageSourceNative), (void **)&this->nativeImageSource);

	ImageBrush^ imageBrush = ref new ImageBrush();
	imageBrush->ImageSource = this->imageSource;

	this->fillTarget->Fill = imageBrush;
	
	this->CreateDXResources();
}

void BlankPage::CreateDXResources()
{
	UINT creationFlags = D3D11_CREATE_DEVICE_BGRA_SUPPORT;

	// This array defines the set of DirectX hardware feature levels this app will support.
    // Note the ordering should be preserved.
    // Don't forget to declare your application's minimum required feature level in its
    // description.  All applications are assumed to support 9.1 unless otherwise stated.
    D3D_FEATURE_LEVEL featureLevels[] = 
    {
        D3D_FEATURE_LEVEL_11_1,
        D3D_FEATURE_LEVEL_11_0,
        D3D_FEATURE_LEVEL_10_1,
        D3D_FEATURE_LEVEL_10_0,
        D3D_FEATURE_LEVEL_9_3,
        D3D_FEATURE_LEVEL_9_2,
        D3D_FEATURE_LEVEL_9_1
    };

	ComPtr<ID3D11Device> device;
    ComPtr<ID3D11DeviceContext> context;
	D3D_FEATURE_LEVEL m_featureLevel;

	D3D11CreateDevice(
				nullptr,                    // specify null to use the default adapter
				D3D_DRIVER_TYPE_HARDWARE,
				0,                          // leave as 0 unless software device
				creationFlags,              // optionally set debug and Direct2D compatibility flags
				featureLevels,              // list of feature levels this app can support
				ARRAYSIZE(featureLevels),   // number of entries in above list
				D3D11_SDK_VERSION,          // always set this to D3D11_SDK_VERSION for Metro style apps
				&device,                    // returns the Direct3D device created
				&m_featureLevel,            // returns feature level of device created
				&context                    // returns the device immediate context
				);

	ComPtr<IDXGIDevice> dxgiDevice;
	device.As(&dxgiDevice);

	this->nativeImageSource->SetDevice(dxgiDevice.Get());

	// create D2D device context
	ComPtr<ID2D1Device> d2dDevice;
    D2D1CreateDevice(dxgiDevice.Get(), NULL, &d2dDevice);
	d2dDevice->CreateDeviceContext(D2D1_DEVICE_CONTEXT_OPTIONS_NONE, &this->d2dContext);
	
	// create brush
	d2dContext->CreateSolidColorBrush(D2D1::ColorF(D2D1::ColorF::Bisque), &this->brush);	
}

/// <summary>
/// Invoked when this page is about to be displayed in a Frame.
/// </summary>
/// <param name="e">Event data that describes how this page was reached.  The Parameter
/// property is typically used to configure the page.</param>
void BlankPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	CompositionTarget::Rendering::add(ref new EventHandler<Object^>(this, &BlankPage::CompositionTarget_Rendering));
}

void BlankPage::CompositionTarget_Rendering(Object^ sender, Object^ e)
{
	this->Draw();
	// this->DrawPolyline();
}

void BlankPage::DrawPolyline()
{
	this->polyLine->Points->Clear();

	float scale = PixelWidth / (float)(PointCount);
	for(int i = 0; i < PointCount; i++)
	{
		Point pt(i * scale, (float)(rand() % PixelHeight));
		this->polyLine->Points->Append(pt);
	}
}

void BlankPage::Draw()
{
	this->BeginDraw();
	
	D2D_POINT_2F pt1, pt2;

	float scale = PixelWidth / (float)(PointCount);
	pt1.x = 0;
	pt1.y = (float)(rand() % PixelHeight);

	for(int i = 1; i < PointCount; i++)
	{
		pt2.x = scale * i;
		pt2.y = (float)(rand() % PixelHeight);

		d2dContext->DrawLine(pt1, pt2, brush.Get(), 2);

		pt1 = pt2;
	}

	this->EndDraw();
}

void BlankPage::BeginDraw()
{
	ComPtr<IDXGISurface> surface;

	RECT updateRect;
	updateRect.left = 0;
	updateRect.top = 0;
	updateRect.right = PixelWidth;
	updateRect.bottom = PixelHeight;

	POINT offset;

	HRESULT beginDrawHR = this->nativeImageSource->BeginDraw(updateRect, &surface, &offset);
	if (beginDrawHR == DXGI_ERROR_DEVICE_REMOVED || beginDrawHR == DXGI_ERROR_DEVICE_RESET)
	{
		// recreate device resources
		CreateDXResources();
		BeginDraw();
	}

	RECT targetRect = 
	{
		offset.x,
		offset.y,
		offset.x + (updateRect.right - updateRect.left),
		offset.y + (updateRect.bottom - updateRect.top)
	};

	// set target
	ComPtr<ID2D1Bitmap1> bitmap;	
    d2dContext->CreateBitmapFromDxgiSurface(surface.Get(), NULL, &bitmap);	
	d2dContext->SetTarget(bitmap.Get());

	d2dContext->BeginDraw();	

	d2dContext->PushAxisAlignedClip(  
		D2D1::RectF(  
			(float)(targetRect.left),  
			(float)(targetRect.top),  
			(float)(targetRect.right),  
			(float)(targetRect.bottom)  
			),  
		D2D1_ANTIALIAS_MODE_ALIASED  
		);

	D2D1::Matrix3x2F offsetMatrix =   
    D2D1::Matrix3x2F::Translation(  
        (float)(targetRect.left),  
        (float)(targetRect.top)  
        );
  
    d2dContext->SetTransform(offsetMatrix);
	d2dContext->Clear(D2D1::ColorF(D2D1::ColorF::Black));
}

void BlankPage::EndDraw()
{
	d2dContext->SetTransform(D2D1::IdentityMatrix());
    d2dContext->PopAxisAlignedClip();

	d2dContext->EndDraw();
	d2dContext->SetTarget(NULL);
	nativeImageSource->EndDraw();
}
